; Auf einen Wert ungleich 0 setzen, wenn das Programm
; auf einem echten Prozessor ausgefhrt werden soll:
.equ RealCPU, 0x00

; Timer0 auf 0xF8 und und Timer1 auf 0xFC, wenn das
; Programm auf einem echten Prozessor laufen soll
.equ Timer0, 0xFC
.equ Timer1, 0xFC

.org 0x100
  ; Port initialisieren
  mov P1,#0xFF

  ;Stift in eine beliebige Ausgangsposition bewegen
  lcall SerialMove

  ; beide Timer in Mode 1 = 16-Bit Timer
  mov 0x89,#0x11 ; TMOD = 0x89
  
  ; Startwerte setzen
  mov TL0,#0x00
  mov TH0,#Timer0
  mov TL1,#0x00
  mov TH1,#Timer1

  ; Timer einschalten und Interrupts freischalten
  mov 0x88,#0x50 ; TCON = 0x88
  mov 0xA8,#0x8A ; IE = 0xA8

loop: ljmp loop


.equ ESC, 0x1B
.equ Zwei, 50
.equ Vier, 52
.equ Sechs, 54
.equ Acht, 56

; Stift steuern durch Eingaben ber 
; die serielle Schnittstelle (2,4,6,8)
; ====================================
SerialMove:
  SerialRead: jnb RI,SerialRead
  mov a,SBUF
  clr RI

  cjne a,#ESC,NotEscape
  ret
  NotEscape:
  
  cjne a,#Zwei,NotZwei
  clr P1.3
  cpl P1.1
  sjmp SerialMove
  NotZwei:

  cjne a,#Vier,NotVier
  clr P1.2
  cpl P1.0
  sjmp SerialMove
  NotVier:

  cjne a,#Sechs,NotSechs
  setb P1.2
  cpl P1.0
  sjmp SerialMove
  NotSechs:

  cjne a,#Acht,SerialMove
  setb P1.3
  cpl P1.1
  sjmp SerialMove


; Verzgerung, so da die Schrittfrequenz 
; nicht mehr als 800Hz betrgt.
; =======================================
Warten:
  push ACC

  ; Nur warten, wenn das Programm nicht im Emulator luft
  mov a,#RealCPU
  jnz DoWait
  pop ACC
  ret
  DoWait:

  mov a,r7
  push ACC
  mov a,r6
  push ACC

  ; 800Hz ungefhr = 2083 Taktschritte 
  ; whle: 2560 = 0x0A00 Taktschritte Pause
  mov r7,#0x0A
  mov r6,#0x00
  sjmp WarteSchleife2
  
WarteSchleife1:
  mov r6,#0xFF
WarteSchleife2:
  nop
  nop
  nop
  djnz r6,WarteSchleife2
  djnz r7,WarteSchleife1

  pop ACC  
  mov r7,0
  pop ACC
  mov r6,a

  pop ACC
  
  ret


; Behandlungsroutinen fr die Timer Interrupts
; ============================================
XSchritt:
  cpl P1.0
  mov TL0,#0x00
  mov TH0,#Timer0
  jb P1.4,ChangeXDir
  jb P1.5,ChangeXDir
  reti  
  ChangeXDir:
  cpl P1.2 
  reti

YSchritt:
  cpl P1.1
  mov TL1,#0x00
  mov TH1,#Timer1
  jb P1.6,ChangeYDir
  jb P1.7,ChangeYDir
  reti  
  ChangeYDir:
  cpl P1.3
  reti

.org 0x0B
  ljmp XSchritt
.org 0x1B
  ljmp YSchritt 
.end
